@@ -43,6 +43,60 @@ class SerializableComponent(ABC):
         serializable (list or dict of primitive types)"""
         raise NotImplemented
 
+    @staticmethod
+    def __none_to_null(obj):
+        """This function iterates over the attributes of the an object and
+        converts all None objects to 'null' to avoid problems with
+        scipy.io.savemat"""
+        if isinstance(obj, dict):
+            for k, v in obj.items():
+                if hasattr(v, '__dict__'):
+                    v = SerializableComponent.__none_to_null(v.__dict__)
+                elif isinstance(v, dict) or isinstance(v, list):
+                    v = SerializableComponent.__none_to_null(v)
+                if v is None:
+                    obj[k] = 'null'
+        elif isinstance(obj, list):
+            for i, v in enumerate(obj):
+                if hasattr(v, '__dict__'):
+                    v = SerializableComponent.__none_to_null(v.__dict__)
+                elif isinstance(v, dict) or isinstance(v, list):
+                    v = SerializableComponent.__none_to_null(v)
+                if v is None:
+                    obj[i] = 'null'
+        return obj
+
+    @staticmethod
+    def __null_to_none(obj):
+        """This function iterates over the attributes of the an object and
+        converts all 'null' objects to None to restore the Python original
+        representation"""
+        if isinstance(obj, dict):
+            for k, v in obj.items():
+                if hasattr(v, '__dict__'):
+                    v = SerializableComponent.__null_to_none(v.__dict__)
+                elif isinstance(v, dict) or isinstance(v, list):
+                    v = SerializableComponent.__null_to_none(v)
+                try:
+                    if v == 'null':
+                        obj[k] = None
+                except ValueError as e:
+                    # Some class do not admit comparison with strings (ndarrays)
+                    pass
+        elif isinstance(obj, list):
+            for i, v in enumerate(obj):
+                if hasattr(v, '__dict__'):
+                    v = SerializableComponent.__null_to_none(v.__dict__)
+                elif isinstance(v, dict) or isinstance(v, list):
+                    v = SerializableComponent.__null_to_none(v)
+                try:
+                    if v == 'null':
+                        obj[i] = None
+                except ValueError as e:
+                    # Some class do not admit comparison with strings (ndarrays)
+                    pass
+        return obj
+
     def save(self, path, data_format=None):
         """Saves the component to the specified format.
 
@@ -99,14 +153,25 @@ class SerializableComponent(ABC):
         with open(path, 'w', encoding=encoding) as f:
             json.dump(self.to_serializable_obj(), f, indent=indent)
 
-    def save_to_mat(self, path):
-        """Save the class in a MATLAB .mat file using scipy. Attributes set to
-        None will be changed to False, since mat data do not support None type
+    def save_to_mat(self, path, avoid_none_objects=True):
+        """Save the class in a MATLAB .mat file using scipy
+
+        Parameters
+        ----------
+        path: str
+            Path to file
+        avoid_none_objects: bool
+            If True, it ensures that all None objects are removed from the
+            object to save to avoid scipy.io.savemat error with this type.
+            Nonetheless, it is computationally expensive, so it is better to
+            leave to False and ensure manually.
         """
         ser_obj = self.to_serializable_obj()
-        for key, value in ser_obj.items():
-            if value is None:
-                ser_obj[key] = 'None'
+        if avoid_none_objects:
+            warnings.warn('Option avoid_none_objects may slow this process. '
+                          'Consider removing None objects manually before '
+                          'calling this function to save time')
+            ser_obj = self.__none_to_null(ser_obj)
         scipy.io.savemat(path, mdict=ser_obj)
 
     def save_to_pickle(self, path, protocol=0):
@@ -162,10 +227,29 @@ class SerializableComponent(ABC):
         return cls.from_serializable_obj(ser_obj_dict)
 
     @classmethod
-    def load_from_mat(cls, path, squeeze_me=True, simplify_cells=True):
+    def load_from_mat(cls, path, squeeze_me=True, simplify_cells=True,
+                      restore_none_objects=True):
+        """Load a mat file using scipy and restore its original class
+
+        Parameters
+        ----------
+        path: str
+            Path to file
+        restore_none_objects: bool
+            If True, it ensures that all 'null' strings are restored as None
+            objects in case that these objects were removed upon saving.
+            Nonetheless, it is computationally expensive, so it is better to
+            leave to False and ensure manually.
+        """
         ser_obj_dict = scipy.io.loadmat(path, squeeze_me=squeeze_me,
                                         simplify_cells=simplify_cells)
-        return cls.from_serializable_obj(ser_obj_dict)
+        if restore_none_objects:
+            warnings.warn('Option restore_none_objects may slow this process. '
+                          'Consider removing "null" strings manually and '
+                          'substitute them for None objects before calling '
+                          'this function to save time')
+            ser_obj_dict = cls.__none_to_null(ser_obj_dict)
+        return cls.from_serializable_obj(cls.__null_to_none(ser_obj_dict))
 
     @classmethod
     def load_from_pickle(cls, path):
@@ -386,13 +470,14 @@ class PickleableComponent(ABC):
 
 class Recording(SerializableComponent):
     """
-    Skeleton class to save the data from one recording. It implements all
-    necessary methods to save and load from several formats. It is composed by 2
-    different  parts, the biosignals recordings (e.g., EEG, MEG), and the
-    experiment data, which saves all the information about the experiment or
-    events. All biosignals and events must be synchronized with the same
-    temporal origin. In order to assure a multiplatform interoperability, this
-    class must be serializable using python primitive types.
+    Class intended to save the data from one recording. It implements all
+    necessary methods to save and load from several formats. It accepts
+    different kinds of data: experiment data, which saves all the information
+    about the experiment (e.g., events); biosignal data (e.g., EEG, MEG, NIRS),
+    bioimaging data (e.g., fMRI, MRI); and custom data (e.g., photos, videos,
+    audio). Temporal data must be must be synchronized with the reference.
+    To assure multiplatform interoperability, this class must be serializable
+    using python primitive types.
     """
     def __init__(self, subject_id, recording_id=None, description=None,
                  source=None, date=None, **kwargs):
@@ -423,51 +508,16 @@ class Recording(SerializableComponent):
         self.source = source
         self.date = date
 
-        # Useful variables
-        self.biosignals = dict()
+        # Data variables
         self.experiments = dict()
+        self.biosignals = dict()
+        self.bioimaging = dict()
+        self.custom_data = dict()
 
         # Set the specified arguments
         for key, value in kwargs.items():
             setattr(self, key, value)
 
-    def add_biosignal(self, biosignal, key=None):
-        """Adds a biosignal recording. Each biosignal has predefined classes
-        that must be instantiated before (e.g., EEG, MEG)
-
-        Parameters
-        ----------
-        biosignal : biosignal class
-            Instance of the biosignal class. This class must be serializable.
-            Current available: EEG, MEG.
-        key: str
-            Custom name for this biosignal. If not provided, the biosignal will
-            be saved in an attribute according to its type in lowercase
-            (e.g., eeg, meg, etc). This parameter is  useful if several
-            biosignals of the same type are added to this recording
-        """
-        # Check errors
-        if not issubclass(type(biosignal), Biosignal):
-            raise TypeError('Parameter biosignal must subclass '
-                            'medusa.io.Biosignal')
-        # Check type
-        biosignal_module_name = type(biosignal).__module__
-        biosignal_class_name = type(biosignal).__name__
-        att = biosignal_class_name.lower() if key is None else key
-        if isinstance(biosignal, CustomBiosignal):
-            warnings.warn('Unspecific biosignal %s. Some high-level functions '
-                          'may not work' % type(biosignal))
-        # Check key
-        if hasattr(self, att):
-            raise ValueError('This recording already contains an attribute '
-                             'with key %s' % att)
-        # Add biosignal
-        setattr(self, att, biosignal)
-        self.biosignals[att] = {
-            'module_name': biosignal_module_name,
-            'class_name': biosignal_class_name
-        }
-
     def add_experiment_data(self, experiment_data, key=None):
         """Adds the experiment data of this recording. Each experiment should
         have a predefined class that must be instantiated before. Several
@@ -508,14 +558,59 @@ class Recording(SerializableComponent):
             'class_name': experiment_class_name
         }
 
+    def add_biosignal(self, biosignal, key=None):
+        """Adds a biosignal recording. Each biosignal has predefined classes
+        that must be instantiated before (e.g., EEG, MEG)
+
+        Parameters
+        ----------
+        biosignal : biosignal class
+            Instance of the biosignal class. This class must be serializable.
+            Current available: EEG, MEG.
+        key: str
+            Custom name for this biosignal. If not provided, the biosignal will
+            be saved in an attribute according to its type in lowercase
+            (e.g., eeg, meg, etc). This parameter is  useful if several
+            biosignals of the same type are added to this recording
+        """
+        # Check errors
+        if not issubclass(type(biosignal), BiosignalData):
+            raise TypeError('Parameter biosignal must subclass '
+                            'medusa.io.BiosignalData')
+        # Check type
+        biosignal_module_name = type(biosignal).__module__
+        biosignal_class_name = type(biosignal).__name__
+        att = biosignal_class_name.lower() if key is None else key
+        if isinstance(biosignal, CustomBiosignalData):
+            warnings.warn('Unspecific biosignal %s. Some high-level functions '
+                          'may not work' % type(biosignal))
+        # Check key
+        if hasattr(self, att):
+            raise ValueError('This recording already contains an attribute '
+                             'with key %s' % att)
+        # Add biosignal
+        setattr(self, att, biosignal)
+        self.biosignals[att] = {
+            'module_name': biosignal_module_name,
+            'class_name': biosignal_class_name
+        }
+
+    def add_bioimaging(self, bioimaging, key=None):
+        # TODO: Create BioimagingData class
+        raise NotImplemented
+
+    def add_custom_data(self, data, key=None):
+        # TODO: Create CustomData class
+        raise NotImplemented
+
     def cast_biosignal(self, key, biosignal_class):
-        """This function casts a biosignal of this recording to the class passed
-        in biosignal_class
+        """This function casts a biosignal to the class passed in
+        biosignal_class
         """
         biosignal_module_name = biosignal_class.__module__
         biosignal_class_name = biosignal_class.__name__
         # Check errors
-        if not issubclass(biosignal_class, Biosignal):
+        if not issubclass(biosignal_class, BiosignalData):
             raise TypeError('Class %s must subclass medusa.io.Biosignal' %
                             biosignal_class_name)
         biosignal = getattr(self, key)
@@ -642,8 +737,8 @@ class Recording(SerializableComponent):
         return cls(**rec_dict)
 
 
-class Biosignal(SerializableComponent):
-    """Skeleton class for biosignals data
+class BiosignalData(SerializableComponent):
+    """Skeleton class for biosignals
     """
 
     @abstractmethod
@@ -661,11 +756,10 @@ class Biosignal(SerializableComponent):
         pass
 
 
-class CustomBiosignal(Biosignal):
-    """Custom experiment data class. This class does not check the arguments and
-    provides less functionality that the proper experiment class. It should
-    only be used for custom experiments that do not fit in other experiment
-    data classes
+class CustomBiosignalData(BiosignalData):
+    """Custom biosignal data class. This class does not check the arguments and
+    provides less functionality that more specific classes. It should
+    only be used for custom signals that do not fit in other data classes
     """
 
     def __init__(self, **kwargs):
@@ -716,7 +810,7 @@ class ExperimentData(SerializableComponent):
 
 class CustomExperimentData(ExperimentData):
     """Custom experiment data class. This class does not check the arguments and
-    provides less functionality that the proper experiment class. It should
+    provides less functionality that a proper experiment class. It should
     only be used for custom experiments that do not fit in other experiment
     data classes
     """
@@ -756,7 +850,7 @@ class ConsistencyChecker(SerializableComponent):
     def add_consistency_rule(self, rule, rule_params, parent=None):
         """Adds a consistency check for the specified attribute It provides 2
         levels of consistency using parameter key, enough to check attributes
-        inside biosignal and experiments classes
+        inside biosignal or experiments classes
 
         Parameters
         ----------
